/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) University of Edinburgh, 1993.                       |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

// $Id: Labels.cc,v 1.2 1998/04/02 13:57:30 npt Exp $

#include <stdio.h>
#include "Octave/Base/ParseContext.h"
#include "Octave/Base/Statements.h"

void CcLabel::define (PTR_STMT st)
{
  stat = st ;
  if (uses)
    {
      StmtListIterator next_use (*uses) ;
      PTR_STMT g ;
      while ((g = next_use()) != 0)
	if (g->stmt_variant() == GOTO_NODE)
	  g->node.fields.bf_ptr1 = st ;
	else
	  fprintf (stderr, "Internal: CcLabel::define\n") ;
    }
}

void CcLabel::add_use (PTR_STMT st)
{
  if (!this)
    return ;
  else
    {
      if (!uses)
	uses = new StmtList ;
      uses->append (st) ;
    }
}

CcLabel* LabelTable::operator[] (PTR_HASH d)
{
  if (d)
    return NameMap<CcLabel>::operator[] ((char*)(d->identifier())) ;
  else
    return 0 ;
}

void LabelTable::add_goto (PTR_HASH labName, PTR_STMT from)
{
  CcLabel *to = (*this)[labName] ;
  if (!to)
    {
      to = new CcLabel (labName->name(), func_stmt, 0) ;
      (*this) += to ;
    }
  to->add_use (from) ;
  from->node.fields.bf_ptr1 = to->labelStmt() ;
}

bool LabelTable::define_label (PTR_HASH labName, PTR_STMT defn)
{
  CcLabel *to = (*this)[labName] ;
  if (to)
    if (to->isDefined())
      return true ;
    else
      {
	to->define (defn) ;
	return false ;
      }
  else
    {
      to = new CcLabel (labName->name(), func_stmt, defn) ;
      (*this) += to ;
      return false ;
    }
}

void LabelTable::check_undefined_labels ()
{
  if (!this)
    return ;

  LabelTable_Iterator next_label (*this) ;
  CcLabel *lab ;

  while (lab = next_label())
    if (!lab->isDefined())
      {
	char ebuf [1024] ;
	char *optr = ebuf ;
	optr += sprintf (optr, "undefined label '%s' used at line", lab->ident()) ; 
	StmtListIterator next_use (*(lab->uses)) ;
	PTR_STMT u1 = next_use() ;
	PTR_STMT u2 = next_use() ;
	if (u2)
	  optr += sprintf (optr, "s") ;
	do
	  {
	    optr += sprintf (optr, " %d", u1->g_line);
	    u1 = u2 ;
	    u2 = next_use() ;
	    if (u2)
	      optr += sprintf (optr, ",") ;
	    else if (u1)
	      optr += sprintf (optr, " and") ;
	  }
	while (u1) ;
	pc->err (ebuf) ;
      }
}


void CaseTable::add_case (int cv, PTR_STMT cs)
{
  CaseLabel *old = (*this)[cv] ;
  CaseLabel **op = &old ;

  if (old)
    {
      while (*op)
	{
	  pc->erri ("`case' label duplicates label on line %d",
		    (*op)->statement()->g_line) ;
	  op = &((*op)->next) ;
	}
      *op = new CaseLabel (cv, cs) ;
    }
  else
    {
      (*this) += new CaseLabel (cv, cs) ;
      ++ncases ;
    }

  if (cv > high)
    high = cv ;

  if (cv < low)
    low = cv ;
}
