/*--------------------------------------------------------------------+
 | OCTAVE - Optimising Compiler Tool And Visualisation Environment    |
 |                                                                    |
 | Copyright (C) University of Edinburgh, 1993.                       |
 |                                                                    |
 | OCTAVE is free software; you can redistribute it and/or modify     |
 | it under the terms of the GNU General Public License as published  |
 | by the Free Software Foundation; either version 1, or (at your     |
 | option) any later version.                                         |
 |                                                                    |
 | OCTAVE is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
 | GNU General Public License for more details.                       |
 |                                                                    |
 | You should have received a copy of the GNU General Public License  |
 | along with OCTAVE; see the file COPYING.  If not, write to         |
 | the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,   |
 | USA.                                                               |
 +--------------------------------------------------------------------*/

// $Id: ParseContext.cc,v 1.15 1999/08/09 13:35:11 npt Exp $

#include "Octave/Base/Sets.h"
#include "Octave/Base/ParseContext.h"


/*--------------------------------------------------------------------+
 |                                                                    |
 |                 parse_constants::parse_constants                   |
 |                                                                    |
 +--------------------------------------------------------------------*/

parse_constants::parse_constants ()
{
  global_unsigned_int = 0 ;
  global_int = 0 ;
  global_unsigned_short_int = 0 ;
  global_unsigned_long_int = 0 ;
  global_long_int = 0 ;
  global_short_int = 0 ;
  global_byte = 0 ;
  global_float = 0 ;
  global_double = 0 ;
  global_long_double = 0 ;
  global_char = 0 ;
  global_signed_char = 0 ;
  global_unsigned_char = 0 ;
  global_wchart = 0 ;
  global_string = 0 ;
  global_bool = 0 ;
  global_complex = 0 ;
  global_gate = 0 ;
  global_event = 0 ;
  global_void = 0 ;
  global_sequence = 0 ;
  global_default = 0 ;
  global_unknown = 0 ;
  global_auto = 0 ;
  global_register = 0 ;
  global_static = 0 ;
  global_extern = 0 ;
  global_explicit = 0 ;
  global_short = 0 ;
  global_long = 0 ;
  global_signed = 0 ;
  global_unsigned = 0 ;
  global_inline = 0 ;
  global_virtual = 0 ;
  global_friend = 0 ;
  global_typedef = 0 ;
  global_const = 0 ;
  global_volatile = 0 ;
  global_mutable = 0 ;
  star_symb = 0 ;
  ellipsis_symb = 0 ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::ParseContext                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

ParseContext::ParseContext (const char* fn, language l = F77) : 
              ScopeStack (), err_list(), fname (fn)
{
  id = nextParseContextID++ ;
  nextSourceFileID = 0 ;
  curSrc = 0 ;
  lang = l ;
  nb_errors = 0 ;
  mod_offset = nowarnflag = 0 ;
  root_symb = 0 ;
  stmt_head = 0 ;
  statement_domain = new Universe<Statement> ;
  expression_domain = new Universe<Expression> ;
  symbol_domain = new Universe<Symbol> ;
  type_domain = new Universe<Datatype> ;
  comment_domain = new Universe<Comment> ;
  trans_domain = new Universe<TransHist> ;
  sym_uses = new ObjectMap<Symbol,SetOf<Expression> > ;
  file_list = new dlist<int,SourceFile> ;
  if (l == F77)
    setSourceFile (fn, 1, 0) ;
}

ParseContext::ParseContext (string& fn, language l = F77) : 
              ScopeStack (), err_list(), fname(fn)
{
  id = nextParseContextID++ ;
  nextSourceFileID = 0 ;
  curSrc = 0 ;
  lang = l ;
  nb_errors = 0 ;
  mod_offset = nowarnflag = 0 ;
  root_symb = 0 ;
  stmt_head = 0 ;
  statement_domain = new Universe<Statement> ;
  expression_domain = new Universe<Expression> ;
  symbol_domain = new Universe<Symbol> ;
  type_domain = new Universe<Datatype> ;
  comment_domain = new Universe<Comment> ;
  trans_domain = new Universe<TransHist> ;
  sym_uses = new ObjectMap<Symbol,SetOf<Expression> > ;
  file_list = new dlist<int,SourceFile> ;
  if (l == F77)
    setSourceFile (fn(), 1, 0) ;
}


/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::~ParseContext                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

ParseContext::~ParseContext() {
  delete statement_domain ;
  delete expression_domain ;
  delete symbol_domain ;
  delete type_domain ;
  delete comment_domain ;
  delete trans_domain ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                      ParseContext::make_stmt                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_STMT ParseContext::make_stmt (statement_tag node_type, PTR_SYMB symb_ptr, 
				  PTR_EXPR ll1, PTR_EXPR ll2,
				  PTR_EXPR ll3, int global_line,
				  int local_line
				  )
{
  PTR_STMT new_stmt = statement_domain->make_member () ;
  new_stmt->set_fields (this, node_type, symb_ptr, ll1, ll2, ll3,
			global_line, local_line) ;
  link_stmt (new_stmt) ;
  return new_stmt ;
}

PTR_STMT ParseContext::make_stmt (statement_tag node_type, PTR_SYMB symb_ptr, 
				  PTR_EXPR ll1, PTR_EXPR ll2,
				  PTR_EXPR ll3)
{
  PTR_STMT new_stmt = statement_domain->make_member () ;
  new_stmt->set_fields (this, node_type, symb_ptr, ll1, ll2, ll3,
			yylineno, yylineno-mod_offset) ;
  link_stmt (new_stmt) ;
  return new_stmt ;
}

PTR_STMT ParseContext::make_stmt (Statement& stat)
{
  PTR_STMT new_stmt = statement_domain->make_member () ;
  new_stmt->set_fields (stat) ;
  link_stmt (new_stmt) ;
  return new_stmt ;
}

PTR_STMT ParseContext::make_stmt ()
{
  PTR_STMT new_stmt = statement_domain->make_member () ;
  new_stmt->set_fields (this, (statement_tag)0, 0, 0, 0, 0,
			0, 0) ;
  return new_stmt ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                      ParseContext::make_expr                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_EXPR ParseContext::make_expr (expression_tag v, PTR_EXPR e1, PTR_EXPR e2, 
				  PTR_SYMB sym)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, v, e1, e2, sym) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (Expression& e)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (e) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (expression_tag v, PTR_FTN_LABEL l)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, v, l) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr ()
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_int_const (PTR_TYPE t, long v)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, v) ;
  new_expr->type = t ;
  new_expr->lvalue = RVALUE ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_char_const (PTR_TYPE t, char c)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, c) ;
  new_expr->type = t ;
  new_expr->lvalue = RVALUE ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_float_const (PTR_TYPE t, string *s)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, s) ;
  new_expr->type = t ;
  new_expr->lvalue = RVALUE ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_string_const (PTR_TYPE t, string *s)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, s) ;
  new_expr->type = t ;
  new_expr->lvalue = LVALUE ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_bool_const (PTR_TYPE t, bool b)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, b) ;
  new_expr->type = t ;
  new_expr->lvalue = RVALUE ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (const char *s)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, s) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (string *s)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, s) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (int i)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, i) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (bool b)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, b) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (float f)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, f) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (double d)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, d) ;
  return new_expr ;
}

PTR_EXPR ParseContext::make_expr (char c)
{
  PTR_EXPR new_expr = expression_domain->make_member () ;
  new_expr->set_fields (this, c) ;
  return new_expr ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::set_ll_list                        |
 |                                                                    |
 |   Construct a list composed from expr object "e" and the           |
 |   parameter "node", making sure that the type of the list node     |
 |   which combines them is "e_type".                                 |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_EXPR ParseContext::set_ll_list (PTR_EXPR node, PTR_EXPR e, 
				    expression_tag e_type) {
  PTR_EXPR join ;
  
  if (!e) {
    // just make an expression node of type e_type from "this"
    join = make_expr (e_type, node, 0, 0) ;
    join->type = global_default ;
    return join ;
  }
  else if (node->expr_variant() != e_type) {
    // we're combining two different expression node types
    join = make_expr (e_type, node,
			  make_expr (e_type, e, 0, 0),
			  0) ;
    join->type = global_default;
    join->node.fields.e2->type = global_default ;
    return join ;
  }
  else {
    join = make_expr (e_type, e, 0, 0) ;
    join->type = global_default ;
    node->append (join) ;
    return node ;
  }
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::append_elist                       |
 |                                                                    |
 |  Append 'node' to the end of 'list' to create an EXPR_LIST list    |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_EXPR ParseContext::append_elist (PTR_EXPR list, PTR_EXPR node)
{
  if (!list) {
    list = make_expr (EXPR_LIST, node, 0, 0) ;
    list->type = global_default ;
    return list ;
  }
  else {
    PTR_EXPR l = list ;
    while (l->node.fields.e2)
      l = l->node.fields.e2 ;
    PTR_EXPR tail = make_expr (EXPR_LIST, node, 0, 0) ;
    l->node.fields.e2 = tail ;
    tail->type = global_default ;
    return list ;
  }
}



/*--------------------------------------------------------------------+
 |                                                                    |
 |                      ParseContext::make_symb                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_SYMB ParseContext::make_symb (symbol_tag node_type, const char *str)
{
  PTR_SYMB new_sym = symbol_domain->make_member () ;
  new_sym->set_fields (this, node_type, str) ;
  link_symb (new_sym) ;
  return new_sym ;
}

PTR_SYMB ParseContext::make_symb (Symbol& s)
{
  PTR_SYMB new_sym = symbol_domain->make_member () ;
  new_sym->set_fields (s) ;
  link_symb (new_sym) ;
  return new_sym ;
}

PTR_SYMB ParseContext::make_symb (symbol_tag node_type, string *s)
{
  PTR_SYMB new_sym = symbol_domain->make_member () ;
  new_sym->set_fields (this, node_type, s) ;
  link_symb (new_sym) ;
  return new_sym ;
}

PTR_SYMB ParseContext::make_symb (symbol_tag node_type, PTR_HASH h, int d)
{
  PTR_SYMB new_sym = symbol_domain->make_member () ;
  new_sym->set_fields (this, node_type, h, d) ;
  link_symb (new_sym) ;
  return new_sym ;
}

PTR_SYMB ParseContext::make_symb ()
{
  PTR_SYMB new_sym = symbol_domain->make_member () ;
  link_symb (new_sym) ;
  return new_sym ;
}

void ParseContext::link_symb (PTR_SYMB s)
{
  if (!root_symb)
    root_symb = last_symb = s ;
  else
    {
      last_symb->thread = s ;
      last_symb = s ;
    }
}

void ParseContext::link_stmt (PTR_STMT s)
{
  if (!stmt_head)
    stmt_head = last_stmt = s ;
  else
    {
      last_stmt->thread = s ;
      last_stmt = s ;
    }
}
/*--------------------------------------------------------------------+
 |                                                                    |
 |                      ParseContext::make_type                       |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_TYPE ParseContext::make_type (type_tag v, int f)
{
  PTR_TYPE new_type = type_domain->make_member () ;
  new_type->set_fields (this, v, f) ;
  return new_type ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                  ParseContext::make_type_spec                      |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_TYPE ParseContext::make_type_spec (int ts)
{
  PTR_TYPE t = type_domain->make_member () ;
  t->set_fields (this, T_DEFAULT) ;
  t->type_flags = ts ;
  t->node.descriptive.base_type = 0 ;
  t->node.descriptive.ranges = 0 ;
  t->node.descriptive.access_flag = 0 ;
  return t ;
}


/*--------------------------------------------------------------------+
 |                                                                    |
 |                      ParseContext::make_label                      |
 |                                                                    |
 | Makes a Fortran label in the current parse context.                |
 +--------------------------------------------------------------------*/

PTR_FTN_LABEL ParseContext::make_label (int l)
{
  PTR_FTN_LABEL new_lab ;
  
  if (l <= 0 || l > 99999) {
    error ("label out of range") ; // BODGE FOR LINENO
    return 0 ;
  }

  // Search for label with integer value 'l':

  new_lab = cur_proc()->node.procedure.labels->get_member (l) ;
  
  if (!new_lab) {
    new_lab = cur_proc()->ftnLabels()->make_member (l) ;
    new_lab->set_fields (this, cur_proc(), l, NO, NO, NO, LABUNKNOWN, 0) ;
  }

  return new_lab ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                     ParseContext::make_comment                     |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_CMNT ParseContext::make_comment (const char *com, int ctype)
{
  PTR_CMNT new_cmnt = comment_domain->make_member () ;
  new_cmnt->set_fields (this, ctype, com) ;
  return new_cmnt ;
}

PTR_CMNT ParseContext::make_comment (string *com, int ctype)
{
  PTR_CMNT new_cmnt = comment_domain->make_member () ;
  new_cmnt->set_fields (this, ctype, com) ;
  return new_cmnt ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::make_transform                     |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_TRANS ParseContext::make_transform (trans_tag t, PTR_STMT s)
{
  PTR_TRANS new_trans = trans_domain->make_member () ;
  new_trans->set_fields (t, s) ;
  warni ("transforming statement by rule %d", t) ;
  return new_trans ;
}

PTR_TRANS ParseContext::make_transform (trans_tag t, PTR_SYMB s)
{
  PTR_TRANS new_trans = trans_domain->make_member () ;
  new_trans->set_fields (t, s) ;
  warni ("transforming symbol by rule %d", t) ;
  return new_trans ;
}

PTR_TRANS ParseContext::make_transform (trans_tag t, PTR_EXPR s)
{
  PTR_TRANS new_trans = trans_domain->make_member () ;
  new_trans->set_fields (t, s) ;
  warni ("transforming expression by rule %d", t) ;
  return new_trans ;
}

PTR_TRANS ParseContext::make_transform (trans_tag t, PTR_TYPE s)
{
  PTR_TRANS new_trans = trans_domain->make_member () ;
  new_trans->set_fields (t, s) ;
  warni ("transforming type definition by rule %d", t) ;
  return new_trans ;
}


/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::make_type_node                     |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_TYPE ParseContext::make_type_node (PTR_TYPE t, PTR_EXPR lengspec)
{
  if (lengspec != 0) {
    t = make_type ( t->type_variant() ) ;
    t->node.fields.ranges = lengspec;
  }
  return t ;
}

/*--------------------------------------------------------------------+
 |                                                                    |
 |                   ParseContext::install_array                      |
 |                                                                    |
 +--------------------------------------------------------------------*/

PTR_TYPE ParseContext::install_array (PTR_EXPR range, PTR_TYPE base, int nd)
{
  PTR_TYPE ret = make_type (T_ARRAY) ;
  ret->node.ar_decl.num_dimensions = nd ;
  ret->node.ar_decl.base_type = base ;
  ret->node.ar_decl.ranges = range ;
  return ret ;
}


/*--------------------------------------------------------------------+
 |                                                                    |
 |                         ParseContext::error                        |
 |                                                                    |
 | The error and warning routines all deposit their messages in a     |
 | list, called ParseContext::msg_list, which is then available to    |
 | the instigator of the parse.                                       |
 |                                                                    |
 | All error and warning routines call 'err' and warn' respectively   |
 | and these routines actually create the list entries.               |
 |                                                                    |
 | The fatal error routines notify the user of an internal error, and |
 | then exit with EXIT_FAILURE status.                                |
 |                                                                    |
 +--------------------------------------------------------------------*/

void ParseContext::err (const char *msg) {
  char buff [2048] ;
  const char *fnm = curSrc->fileName() ;
  char *trace = curSrc->include_trace() ;
  sprintf (buff, "Error: line %d of %s; %s\n%s", yylineno, fnm, msg, trace) ;
  delete [] trace ;
  err_list.append ( new string (buff) ) ;
  ++ nb_errors ;
}

void ParseContext::error (const char *msg) {
  err (msg) ;
}

void ParseContext::erri (const char *fmt, int arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  err (buff) ;
}

void ParseContext::errstr (const char *fmt, const char *arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  err (buff) ;
}

void ParseContext::warn (const char *msg) {
  if (nowarnflag)
    return ;
  else {
    char buff [2048] ;
    const char *fnm = curSrc->fileName() ;
    char *trace = curSrc->include_trace() ;
    sprintf (buff, "Warning: line %d of %s; %s\n%s",
	     yylineno, fnm, msg, trace) ;
    err_list.append ( new string (buff) ) ;
  }
}

void ParseContext::warni (const char* fmt, int arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  warn (buff) ;
}

void ParseContext::warnstr (const char *fmt, char *arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  warn (buff) ;
}

void ParseContext::fatal (const char *) {
  fprintf (stderr, "Internal parser error: %s\n") ;
  exit (EXIT_FAILURE) ;
}

void ParseContext::fatali (const char* fmt, int arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  fatal (buff) ;
}

void ParseContext::fatalstr (const char *fmt, const char *arg) {
  char buff [256] ;
  sprintf (buff, fmt, arg) ;
  fatal (buff) ;
}

// Initialise the static data members from parse_constants class

string   parse_constants::star_string ("*") ;
string   parse_constants::unit_string ("unit") ;
string   parse_constants::dbl_star_string ("**") ;
string   parse_constants::fmt_string ("fmt") ;
string   parse_constants::nml_string ("nml") ;
string   parse_constants::ellipsis_string ("...") ;
int      ParseContext::nextParseContextID = 0 ;
