/*
 * EOS Timers (Loop or other level) Windows (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>

static int timers_max=0;

static double* Timers;
static double* Timers1;
static long*   Calls;

static int first_run=0;

void eos_timers_init(int timers_max1)
{
  long x;
  long y;
  long z;

  z=timers_max1+1;
  timers_max=timers_max1;

  x=(sizeof(double)*z);
  printf("FATOM: Allocating %u bytes of memory [Timers]...\n", x);

  Timers=(double *) calloc(z, sizeof(double));

  if (Timers == NULL) 
  {
    printf("\nFatal Error: Can't allocate memory!\n");
    exit(1);
  }

  for (y=1; y<=timers_max; y++)
  {
    Timers[y]=0;
  }

  x=(sizeof(double)*z);
  printf("FATOM: Allocating %u bytes of memory [Timers1]...\n", x);

  Timers1=(double *) calloc(z, sizeof(double));

  if (Timers1 == NULL) 
  {
    printf("\nFatal Error: Can't allocate memory!\n");
    exit(1);
  }

  for (y=1; y<=timers_max; y++)
  {
    Timers1[y]=(double) clock() / CLOCKS_PER_SEC;
  }

  x=(sizeof(long)*z);
  printf("FATOM: Allocating %u bytes of memory [Calls]...\n", x);

  Calls=(long *) calloc(z, sizeof(long));

  if (Calls == NULL) 
  {
    printf("\nFatal Error: Can't allocate memory!\n");
    exit(1);
  }

  for (y=1; y<=timers_max; y++)
  {
    Calls[y]=0;
  }

  printf("\nFATOM: Allocated successfully!\n\n");
}

void eos_timers_finish(void)
{
  FILE *f;
  int i=0;

  unsigned int val=0;
  int q=0;

  double time_all=0;

  printf("\nFATOM: Saving collected data ...\n");

  if ((f=fopen("_eos_llprof_out","wt"))==NULL)
  {
    printf("\nFatal Error (FATOM): Can't write file!\n");
    exit(1);
  }
         
  for (i=1; i<=timers_max; i++)
  {
    fprintf(f, "timer={%5u}, time={%7.1f}, calls={%8u}\n", 
               i, 
               Timers[i],
               Calls[i]);
    time_all+=Timers[i];
  }

  fprintf(f, "\noverall time = %.1f sec.", time_all);
   
  fclose(f);

  printf("FATOM (eos_mpc_fb1): Data saved successfully!\n");

  return;
}

void EOS_TIMER1(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  Timers1[tmr]=(double) clock() / CLOCKS_PER_SEC;
}

void EOS_TIMER2(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }

  tmr=*tmr1;

  Timers[tmr]+=((double) clock() / CLOCKS_PER_SEC) - Timers1[tmr];
}

void eos_timer1(int tmr)
{
  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  Calls[tmr]++;

  Timers1[tmr]=(double) clock() / CLOCKS_PER_SEC;
}

void eos_timer2(int tmr)
{
  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }

  Timers[tmr]+=((double) clock() / CLOCKS_PER_SEC) - Timers1[tmr];
}

void eos_timer1_(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  Timers1[tmr]=(double) clock() / CLOCKS_PER_SEC;
}

void eos_timer2_(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }

  tmr=*tmr1;

  Timers[tmr]+=((double) clock() / CLOCKS_PER_SEC) - Timers1[tmr];
}

void eos_timer1__(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  Timers1[tmr]=(double) clock() / CLOCKS_PER_SEC;
}

void eos_timer2__(int *tmr1)
{
  int tmr=0;

  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }

  tmr=*tmr1;

  Timers[tmr]+=((double) clock() / CLOCKS_PER_SEC) - Timers1[tmr];
}

void _Z10eos_timer1i(int tmr)
{
  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  Calls[tmr]++;

  Timers1[tmr]=(double) clock() / CLOCKS_PER_SEC;
}

void _Z10eos_timer2i(int tmr)
{
  if (first_run==0)
  {
    eos_timers_init($EOS1$);
    atexit(eos_timers_finish);
    first_run=1;
  }

  Timers[tmr]+=((double) clock() / CLOCKS_PER_SEC) - Timers1[tmr];
}
