/*
 * EOS Timers (Loop or other level) Windows (analysis)
 *
 * Copyright (C) 2000-2004 by Grigori G. Fursin
 *
 * homepage: http://homepages.inf.ed.ac.uk/gfursin
 *
 * ICSA, School of Informatics,
 * University of Edinburgh, UK
 */

#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>

#include "papi.h"

#define timers_max   $EOS1$
#define queue_max    $EOS2$
#define PAPI_event   $EOS3$
#define file_out     "_eos_llprof_out.$EOS3$"

int retval;
int EventSet=PAPI_NULL;

long_long values;

static double      Timers[timers_max+1];
static double      Timers1[timers_max+1];
static long_long   Timers2[timers_max+1];
static long_long   Calls[timers_max+1];

static int first_run=0;

void eos_timers_init(void)
{
  long y;
  FILE *f;

  for (y=1; y<=timers_max; y++)
  {
    Timers[y]=0;
    Timers1[y]=0;
    Timers2[y]=0;
    Calls[y]=0;  
  }

  if ((f=fopen(file_out,"wt"))==NULL)
  {
    printf("\nFatal Error (FATOM): Can't write file!\n");
    exit(1);
  }
  fclose(f);

   if ((retval = PAPI_library_init(PAPI_VER_CURRENT)) != PAPI_VER_CURRENT)
   {
     printf("\nError: PAPI_library_init");
     exit(1);
   }

   if ((retval = PAPI_create_eventset(&EventSet)) != PAPI_OK)
   {
     printf("\nError: PAPI get events");
     exit(1);
   }

  if (PAPI_add_event(EventSet, PAPI_event) != PAPI_OK) 
  {
     printf("\nError: can't add PAPI event");
     exit(1);
  }

  if ((retval = PAPI_start(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't start PAPI");
     exit(1);
  }
}

void save_timers(void)
{
  FILE *f;
  int i=0;

  unsigned int val=0;

  double time_all=0;

  if ((f=fopen(file_out,"wt"))==NULL)
  {
    printf("\nFatal Error (FATOM): Can't write file!\n");
    exit(1);
  }
         
  for (i=1; i<=timers_max; i++)
  {
    fprintf(f, "timer={%5u}, time={%7.2f}, calls={%15lld}, events={%15lld}\n", 
               i, Timers[i], Calls[i], Timers2[i]);

    time_all+=Timers[i];
  }

  fprintf(f, "\noverall time = %.2f sec.\n", time_all);
     
  fclose(f);
}

void eos_timers_finish(void)
{
  save_timers();
  
  return;
}

void EOS_TIMER1(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  if ((retval = PAPI_reset(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't reset PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers1[tmr]=tt;
}

void EOS_TIMER2(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if ((retval = PAPI_read(EventSet,&values)) != PAPI_OK)
  {
     printf("\nError: can't read PAPI event");
     exit(1);
  }

  tmr=*tmr1;
  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers[tmr]+=tt - Timers1[tmr];
  Timers2[tmr]+=values;
}

void eos_timer1(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  if ((retval = PAPI_reset(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't reset PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers1[tmr]=tt;
}

void eos_timer2(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if ((retval = PAPI_read(EventSet,&values)) != PAPI_OK)
  {
     printf("\nError: can't read PAPI event");
     exit(1);
  }

  tmr=*tmr1;
  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers[tmr]+=tt - Timers1[tmr];
  Timers2[tmr]+=values;
}

void eos_timer1_(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  if ((retval = PAPI_reset(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't reset PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers1[tmr]=tt;
}

void eos_timer2_(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if ((retval = PAPI_read(EventSet,&values)) != PAPI_OK)
  {
     printf("\nError: can't read PAPI event");
     exit(1);
  }

  tmr=*tmr1;
  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers[tmr]+=tt - Timers1[tmr];
  Timers2[tmr]+=values;
}

void eos_timer1__(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  tmr=*tmr1;

  Calls[tmr]++;

  if ((retval = PAPI_reset(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't reset PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers1[tmr]=tt;
}

void eos_timer2__(int *tmr1)
{
  int tmr=0;
  double tt=0;
  double tt1=0;

  if ((retval = PAPI_read(EventSet,&values)) != PAPI_OK)
  {
     printf("\nError: can't read PAPI event");
     exit(1);
  }

  tmr=*tmr1;
  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers[tmr]+=tt - Timers1[tmr];
  Timers2[tmr]+=values;
}

void _Z10eos_timer1i(int tmr)
{
  double tt=0;
  double tt1=0;

  if (first_run==0)
  {
    eos_timers_init();
    atexit(eos_timers_finish);
    first_run=1;
  }
  
  Calls[tmr]++;

  if ((retval = PAPI_reset(EventSet)) != PAPI_OK)
  {
     printf("\nError: can't reset PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers1[tmr]=tt;
}

void _Z10eos_timer2i(int tmr)
{
  double tt=0;
  double tt1=0;

  if ((retval = PAPI_read(EventSet,&values)) != PAPI_OK)
  {
     printf("\nError: can't read PAPI event");
     exit(1);
  }

  tt=((double) PAPI_get_real_usec())*0.000001;
  Timers[tmr]+=tt - Timers1[tmr];
  Timers2[tmr]+=values;
}

