//
// "Edinburgh Optimizing Software" Application Checking Server
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.net.*;
import java.io.*;

//Main class
public class server
{
  // *******************************************************************
  public static void main(String args[]) 
  {
    long misc;
    String FileOpt="server.cl";
    String FileCFG="server.cfg";
    
    //***********************************************************************
    System.out.println("EOS AC Server   V1.07   (C) by Grigori G.Fursin   December,2004");

    //Declaration of class with global variables
    CGlobal glob = new CGlobal();

    //Declaration of class for command-line parsing
    glob.clp = new CCLParse();

    if (glob.clp.Init(10, 0, FileOpt)!=0)
    //                 |  |  | 
    //                 |  |  |-------------- Name of File with Options
    //                 |  |
    //                 |  |----------------- 1 - Ignore case, 0 - Don't ignore case
    //                 |
    //                 |-------------------- Max Number of Options
    {
      System.out.println("\nError: Can't initialize object CCLParse!");
      return;
    }

    misc=0;
  
    misc =glob.clp.Add("--help", "-h", "-?", "", 0);     //Param 0
    misc+=glob.clp.Add("--version", "-v", "", "", 0);    //Param 1
    misc+=glob.clp.Add("--verbose", "-vb", "", "", 0);   //Param 2
    misc+=glob.clp.Add("--port", "-p", "", "4855", 1);   //Param 3
    misc+=glob.clp.Add("--nofork", "-nf", "", "", 0);    //Param 4

    if (misc!=0)
    {
      System.out.println("\nError: Internal, while initializing CCLParse!");
      return;
    }

    misc=glob.clp.Parse(args);
    if (misc==1)
    {
      System.out.println("\nError: Unknown option in command line!");
      return;
    }
    else if (misc==2)
    {
      System.out.println("\nError: Option can't be used without parameter in command line!");
      return;
    }
    else if (misc==3)
    {
      System.out.println("\nError: Option can't be used without parameter in file with options!");
      return;
    }
    else if (misc==4)
    {
      System.out.println("\nError: Unknown option in file with options!");
      return;
    }
    else if (misc>0)
    {
      System.out.println("\nError: Internal, while parsing command line!");
      return;
    }

    if (glob.clp.GetParamLong(glob.paramVersion)==1)
    {
      System.out.println("");
      System.out.println("Version information:");
      System.out.println("  Version 1.07, released on 2004.December.20");

      return;
    }
    
    if (glob.clp.GetParamLong(glob.paramHelp)==1)
    {
      //Help
      System.out.println("");
      System.out.println("Brief information:");
      System.out.println("  This is EOS AC Server");
      System.out.println("");
      System.out.println("  Copyright (C) 2000-2004 by Grigori G.Fursin");
      System.out.println("  http://homepages.inf.ed.ac.uk/gfursin");
      System.out.println("");
      System.out.println("  ICSA, School of Informatics,");
      System.out.println("  University of Edinburgh, UK");
      System.out.println("");
      System.out.print  ("Options (you can also add options into file ");
      System.out.print(FileOpt);
      System.out.println("):");
      System.out.println("  --help, -h, -?             Help;");
      System.out.println("  --version, -v              Print version;");
      System.out.println("  --verbose, -vb             Print additional diagnostic messages;");
      System.out.println("  --port, -p <port number>   Port number for server [Default=4855];");
      System.out.println("  --nofork, -nf              No fork for child process;");

      return;
    }

    if (glob.clp.GetParamLong(glob.paramVerbose)==1)
    {                                              
      System.out.println("");
      System.out.println("Options:");
      System.out.print(" Verbose=");
      System.out.println(glob.clp.GetParamLong(glob.paramVerbose));
      System.out.print(" Port   =");
      System.out.println(glob.clp.GetParamLong(glob.paramPort));
      System.out.print(" NoFork =");
      System.out.println(glob.clp.GetParamLong(glob.paramNofork));
    }

    //Declaration of class for configuration file parsing
    glob.cfg = new CConfig();

    if (glob.cfg.Init(32, 0, FileCFG, 128)!=0)
    //                 |  |        |    |
    //                 |  |        |    |--- Max Number of Array Parameters
    //                 |  |        | 
    //                 |  |        |-------- Name of File with Parameters
    //                 |  |
    //                 |  |----------------- 1 - Ignore case, 0 - Don't ignore case (for value)
    //                 |
    //                 |-------------------- Max Number of Parameters
    {
      System.out.println("\nError: Can't initialize object CConfig!");
      return;
    }

    misc=0;
  
    misc+=glob.cfg.Add("c_server_host",      "",  1);  //Param  0
    misc+=glob.cfg.Add("c_server_port",      "",  1);  //Param  1
    misc+=glob.cfg.Add("path_local",         "",  1);  //Param  2
    misc+=glob.cfg.Add("separator",          "",  1);  //Param  3
    misc+=glob.cfg.Add("eos_timers_d",       "",  1);  //Param  4
    misc+=glob.cfg.Add("eos_timers_f",       "",  1);  //Param  5
    misc+=glob.cfg.Add("eos_timers_m",       "",  1);  //Param  6
    misc+=glob.cfg.Add("eos_timers_m2",      "",  1);  //Param  7
    misc+=glob.cfg.Add("eos_timers_m3",      "",  1);  //Param  8
    misc+=glob.cfg.Add("eos_timers_p",       "",  1);  //Param  9
    misc+=glob.cfg.Add("eos_timers_cl_f77",  "",  1);  //Param 10
    misc+=glob.cfg.Add("eos_timers_cl_f77x", "",  1);  //Param 11
    misc+=glob.cfg.Add("eos_timers_cl_f77o", "",  1);  //Param 12
    misc+=glob.cfg.Add("eos_timers_cl_f90",  "",  1);  //Param 13
    misc+=glob.cfg.Add("eos_timers_cl_f90x", "",  1);  //Param 14
    misc+=glob.cfg.Add("eos_timers_cl_f90o", "",  1);  //Param 15
    misc+=glob.cfg.Add("eos_timers_cl_c",    "",  1);  //Param 16
    misc+=glob.cfg.Add("eos_timers_cl_cx",   "",  1);  //Param 17
    misc+=glob.cfg.Add("eos_timers_cl_co",   "",  1);  //Param 18
    misc+=glob.cfg.Add("eos_timers_cl_cpp",  "",  1);  //Param 19
    misc+=glob.cfg.Add("eos_timers_cl_cpp1", "",  1);  //Param 20
    misc+=glob.cfg.Add("eos_timers_cl_cppx", "",  1);  //Param 21
    misc+=glob.cfg.Add("eos_timers_cl_cppo", "",  1);  //Param 22
    misc+=glob.cfg.Add("eos_papi_l",         "",  1);  //Param 23
    misc+=glob.cfg.Add("eos_papi_c",         "",  1);  //Param 24
    misc+=glob.cfg.Add("papievent"         , "", 32);  //Param 25

    if (misc!=0)
    {
      System.out.println("\nError: Internal, while initializing CConfig!");
      return;
    }

    misc=glob.cfg.Parse();
    if (misc==1)
    {
      System.out.println("\nError: Can't find configuration file!");
      return;
    }
    else if (misc==2)
    {
      System.out.println("\nError: Unknown syntax in configration file!");
      return;
    }
    else if (misc==3)
    {
      System.out.println("\nError: Unknown syntax in configration file!");
      return;
    }
    else if (misc==4)
    {
      System.out.println("\nError: Unknown parameter in configuration file!");
      return;
    }
    else if (misc==5)
    {
      System.out.println("\nError: IO Problems while parsing configuration file!");
      return;
    }
    else if (misc>0)
    {
      System.out.println("\nError: Internal, while parsing configuration file!");
      return;
    }

    //Preparing Server
    ServerSocket sockfd = null;

    try 
    {
      sockfd = new ServerSocket(glob.clp.GetParamInt(glob.paramPort));
    } 
    catch (IOException e) 
    {
      System.out.println("\nError: Can't start listening!");
      return;
    }

    //Start listening
    System.out.println("");
    System.out.print("Server is waiting for connections [port ");
    System.out.print(glob.clp.GetParamLong(glob.paramPort));
    System.out.println("] ...");

    System.out.println("");

    for (;;)
    {
      Socket newsockfd = null;
      try 
      {
        newsockfd = sockfd.accept();
      } 
      catch (IOException e) 
      {
        System.out.println("Error: Accept error!");
//      return ; // don't stop - may be temporal error
        continue;
      }
    
      if (glob.clp.GetParamLong(glob.paramNofork)==0)
      {
        //If Fork

        glob.proc_num++;
        System.out.print("[");
        System.out.print(glob.proc_num);
        System.out.println("] Child process activated ...");

	    new serverThreadProc(newsockfd, glob).start();
      }
      else
      {
        // If No Fork

        System.out.println("Connection established ...");

        new ProcessMessages (newsockfd, glob);
        try
        {
          newsockfd.close();
        }
        catch (IOException e) 
        {
          System.out.println("Error: Can't close socket connection!");
        }

        System.out.println("Connection closed ...");
      }
    }
  }
}
