//
// Class CConfig for configuration file parsing
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CConfig
{
  //Variables
  private int XNumParamMax;
  private int XNumArrayMax;
  private int XIgnoreCase;
  private String XFileNameParam;
 
  private String[] Param;
  private String[] Value;
  private int[]    Pnt;
  private int[]    MaxNumValues;
  private int[]    CurPrm;

  private int CurParam;
  private int CurParam1;

  //****************************************************************************
  //"Initialization"
  //
  //Input:  NumParamMax   = Max Number of Parameters
  //        IgnoreCase    = 1 - Ignore case, 0 - Don't ignore case (for Value!)
  //        FileNameParam = Name of file with parameters
  //        NumArrayMax   = Max Number of Array Parameters
  //
  //Return: 0  - Ok
  //        >0 - Error
  //

  public int Init(int NumParamMax,
                  int IgnoreCase,
                  String FileNameParam,
                  int NumArrayMax)
  {
    int x;

    XNumParamMax=NumParamMax;
    XNumArrayMax=NumArrayMax;
    XIgnoreCase=IgnoreCase;    
    XFileNameParam=FileNameParam;

    if (NumParamMax<1) return 1;

    if ((IgnoreCase!=0) && (IgnoreCase!=1)) return 1;

    Param       =new String [XNumParamMax];
    Value       =new String [XNumArrayMax];
    Pnt         =new int    [XNumParamMax];
    MaxNumValues=new int    [XNumParamMax];
    CurPrm      =new int    [XNumParamMax];

    for (x=0; x<XNumParamMax; x++) 
    {
      Param       [x]="";
      Pnt         [x]=0;
      MaxNumValues[x]=0;
      CurPrm      [x]=0;
    }

    for (x=0; x<XNumArrayMax; x++) 
    {
      Value[x]="";
    }

    CurParam =0;
    CurParam1=0;

    return 0;
  }

  //****************************************************************************
  //"Add Option"
  //
  //Input:  SParam    = Parameter
  //        Default   = Default Value
  //        MaxNumVal = Max Number of acceptable values (1, ...)
  //
  //Return: 0  - Ok
  //        >0 - Error
  //
  public int Add(String SParam, String Default, int MaxNumVal)
  {
    if ((CurParam>XNumParamMax) ||
        (CurParam1+MaxNumVal>XNumArrayMax) ||
        (MaxNumVal<1)) return 1;

    Param[CurParam]=SParam;

    MaxNumValues[CurParam]=MaxNumVal;
    Pnt[CurParam]=CurParam1;
    CurPrm[CurParam]=0;

    Value[CurParam1]=Default;

    if (XIgnoreCase==1)
    {
      Value[CurParam1]=Value[CurParam1].toLowerCase(); 
    }

    CurParam++;
    CurParam1+=MaxNumVal;

    return 0;
  }

  //****************************************************************************
  //"Parse Configuration File"
  //
  //Return: 0  - Ok
  //        >0 - Error
  //
  public int Parse()
  {
    BufferedReader fin=null;

    try
    {
      fin=new BufferedReader(new FileReader(XFileNameParam));
    }
    catch (IOException e2)
    {
      return 1;
    }

    String str1="";

    try
    {
      while ((str1=fin.readLine())!=null)
      {
        str1=str1.trim();
        if ((!str1.equals("")) && (str1.charAt(0)!='#'))
        {
          int i1=str1.indexOf('=');
          int i2=str1.length();

          if (i1<0) return 2;
          if (i1==i2-1) return 3;
          
          String str2=str1.substring(0, i1);
          str2=str2.trim();
          str2=str2.toLowerCase();

          String str3=str1.substring(i1+1, i2);
          str3=str3.trim();
          if (XIgnoreCase==1) str3=str3.toLowerCase();
          
          boolean found=false;
          for (int i=0; i<CurParam; i++)
          {
            if (Param[i].equals(str2))
            {
              CurPrm[i]++;
              if (CurPrm[i]>MaxNumValues[i]) return 6;

              Value[Pnt[i]+CurPrm[i]-1]=str3;
              found=true;
            }
          }

          if (!found) return 4;
        }
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      return 5;
    }

    try
    {
      fin.close();
    }
    catch (Exception e3)
    {}
      
    return 0;
  }

  //****************************************************************************
  //"Get Max Number of Values"
  //
  public int GetMaxNumVal(int NumParam)
  {   
    if ((NumParam<0) || (NumParam>=CurParam)) return 0;

//    return MaxNumValues[NumParam];
    return CurPrm[NumParam];
  }

  //****************************************************************************
  //"Get String Parameter"
  //
  public String GetParamStr(int NumParam, int Shift)
  {   
    if ((NumParam<0) || (NumParam>=CurParam)) return null;

    return Value[Pnt[NumParam]+Shift];
  }

  //****************************************************************************
  //"Get Long Parameter"
  //
  public long GetParamLong(int NumParam, int Shift)
  {
    if ((NumParam<0) || (NumParam>=CurParam)) return 0;

    long ln=0;
    try
    {
      ln=Long.valueOf(Value[Pnt[NumParam]+Shift]).longValue();
    }
    catch (NumberFormatException nfe1)
    {}
    
    return ln;
  }

  //****************************************************************************
  //"Get Int Parameter"
  //
  public int GetParamInt(int NumParam, int Shift)
  {
    if ((NumParam<0) || (NumParam>=CurParam)) return 0;

    int in=0;
    try
    {
      in=Integer.valueOf(Value[Pnt[NumParam]+Shift]).intValue();
    }
    catch (NumberFormatException nfe1)
    {}
    
    return in;
  }

  //****************************************************************************
  //"Get String Parameter"
  //
  public String GetParamStr(int NumParam)
  {   
    if ((NumParam<0) || (NumParam>=CurParam)) return null;

    return Value[Pnt[NumParam]];
  }

  //****************************************************************************
  //"Get Long Parameter"
  //
  public long GetParamLong(int NumParam)
  {
    if ((NumParam<0) || (NumParam>=CurParam)) return 0;

    long ln=0;
    try
    {
      ln=Long.valueOf(Value[Pnt[NumParam]]).longValue();
    }
    catch (NumberFormatException nfe1)
    {}
    
    return ln;
  }

  //****************************************************************************
  //"Get Int Parameter"
  //
  public int GetParamInt(int NumParam)
  {
    if ((NumParam<0) || (NumParam>=CurParam)) return 0;

    int in=0;
    try
    {
      in=Integer.valueOf(Value[Pnt[NumParam]]).intValue();
    }
    catch (NumberFormatException nfe1)
    {}
    
    return in;
  }

  //****************************************************************************
  //"Set String Parameter"
  //
  public void SetParamStr(int NumParam, String str)
  {   
    if ((NumParam<0) || (NumParam>=CurParam)) return;

    Value[Pnt[NumParam]]=str;
  }

  //****************************************************************************
  //"Save Configuration"
  //
  //Return: 0  - Ok
  //        >0 - Error
  //
  public int Save()
  {
    BufferedWriter fout=null;

    try
    {
      fout=new BufferedWriter(new FileWriter(XFileNameParam));
    }
    catch (IOException e2)
    {
      return 1;
    }

    String str1="";

    try
    {
      for (int i=0; i<CurParam; i++)
      {
        String name=Param[i];
        String param=GetParamStr(i);

        if (!param.equals(""))
        {
          fout.write(name+" = "+param);
          fout.newLine();
        }
      }
    }
    catch (Exception e1)
    {
      try
      {
        fout.close();
      }
      catch (Exception e2)
      {}
      return 5;
    }

    try
    {
      fout.close();
    }
    catch (Exception e3)
    {}
      
    return 0;
  }
}
