//
// Class CLLP for parsing loop level profile
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CLLP
{
  //Variables (public only for easy copying - do not access them directly)
  public String fn;
  public int nl=0;

  private String err="";   

  public String[] tim;
  public String[] cal;

  public double[]  timx_min;
  public double[]  timx_diff;
  public double[]  timx_diff1;
  public boolean[] timx_better;
  public boolean[] timx_sel;

  //****************************************************************************
  //"Initialization"
  //
  //Input:  filename = name with path for loop level profile
  //
  public void Init(String filename)
  {
    fn=filename;
  }

  //****************************************************************************
  public void Delete()
  {
    //Delete llp profile if exist
    File fdel=null;
    try
    {
      fdel=new File(fn);
      fdel.delete();
    }
    catch (NullPointerException e1)
    {}
  }

  //****************************************************************************
  public int Parse()
  {
    //Read time    
    BufferedReader fin;
    String str;

    try
    {
      fin=new BufferedReader(new FileReader(fn));
    }
    catch (IOException e1)
    {
      err="Error: Can't find file with LLP file!";
      return -1; 
    }

    nl=0;

    try
    {
      while ((str=fin.readLine())!=null)
      {
        if (str.indexOf("timer={")>=0) nl++;
      }
    }
    catch (Exception e2)
    {
      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}
      err="Error: Problems while reading file with LLP file!";
      return -2;
    }

    try
    {
      fin.close();
    }
    catch (IOException e5)
    {
      err="Error: Can't close file with LLP file!";
      return -3; 
    }

    tim=new String [nl];
    cal=new String [nl];

    timx_min   =new double[nl];
    timx_diff  =new double[nl];
    timx_diff1 =new double[nl];
    timx_better=new boolean[nl];
    timx_sel   =new boolean[nl];

    nl=0;

    try
    {
      fin=new BufferedReader(new FileReader(fn));
    }
    catch (IOException e1)
    {
      err="Error: Can't find file with LLP file!";
      return -1; 
    }

    try
    {
      while ((str=fin.readLine())!=null)
      {
        String tim1="";
        String cal1="";

        int i1=0;
        int i2=0;

        i1=str.indexOf("time={");
        if (i1!=-1)
        {
          i2=str.indexOf("}", i1+5);
          if (i2!=-1)
          {
            tim1=str.substring(i1+6, i2);

            i1=str.indexOf("calls={");
            if (i1!=-1)
            {
              i2=str.indexOf("}", i1+6);
              if (i2!=-1)
              {
                cal1=str.substring(i1+7, i2);

                tim[nl]=tim1;
                cal[nl]=cal1;

                double t1=0; 
                try {t1=Double.valueOf(tim1).doubleValue();}
                catch (NumberFormatException nfe1){}

                timx_min[nl]=t1;
                timx_diff[nl]=0;
                timx_diff1[nl]=0;
		timx_better[nl]=false;
		timx_sel[nl]=false;

                nl++;
              }
            }
          }
        }
      }
    }
    catch (Exception e2)
    {
      try
      {
        fin.close();
      }
      catch (Exception e3)
      {}
      err="Error: Problems while reading file with LLP file!";
      return -2;
    }

    try
    {
      fin.close();
    }
    catch (IOException e5)
    {
      err="Error: Can't close file with LLP file!";
      return -3; 
    }

    return 0;
  }

  //****************************************************************************
  public int Copy(String add)
  {
    //Read time    
    BufferedReader fin;
    BufferedWriter fout;
    String str;

    try
    {
      fin=new BufferedReader(new FileReader(fn));
    }
    catch (IOException e1)
    {
      err="Error: Can't find file with LLP file!";
      return -1; 
    }

    try
    {
      fout=new BufferedWriter(new FileWriter(fn+add));
    }
    catch (IOException e1)
    {
      err="Error: Can't open file with LLP file to copy!";
      return -4; 
    }

    try
    {
      while ((str=fin.readLine())!=null)
      {
        fout.write(str);
        fout.newLine();
      }
    }
    catch (Exception e2)
    {
      try
      {
        fin.close();
        fout.close();
      }
      catch (Exception e3)
      {}
      err="Error: Problems while reading/writing file(s) with LLP file!";
      return -2;
    }

    try
    {
      fin.close();
      fout.close();
    }
    catch (IOException e5)
    {
      err="Error: Can't close file(s) with LLP file!";
      return -3; 
    }

    return 0;
  }

  //****************************************************************************
  public int GetNumberOfTimers()
  {
    return nl;
  }

  //****************************************************************************
  public String PrintLine(int n)
  {
    String str="";
    String str1=String.valueOf(n+1);

    if ((n>=0) && (n<nl))
    {
      int i1=str1.length();
    
      for (int i=i1; i<5; i++)
      {
        str1="0"+str1;
      }

      str=str1+") time={"+
          tim[n]+"}, calls={"+cal[n]+"}";
    }

    return str;
  }

  //****************************************************************************
  public void Comp(CLLP cllp)
  {
    int i=0;

    int n1=cllp.GetNumberOfTimers();
    int n2=n1;
    if (nl<n2) n2=nl;

    for (i=0; i<n2; i++)
    {
      String t=cllp.GetTime(i);

      double t1=0; 
      try {t1=Double.valueOf(t).doubleValue();}
      catch (NumberFormatException nfe1){}

      double t2=0; 
      try {t2=Double.valueOf(tim[i]).doubleValue();}
      catch (NumberFormatException nfe1){}

      timx_better[i]=false;
      if ((t1!=0) && (t1<timx_min[i]) && (timx_sel[i]))
      {
        timx_diff[i]=t2-t1;
        timx_diff1[i]=timx_min[i]-t1;
        timx_better[i]=true;
        timx_min[i]=t1;
      }
    }
  }

  //****************************************************************************
  public void select(int i)
  {
    timx_sel[i]=true;
  }

  //****************************************************************************
  public String GetTime(int i)
  {
    return tim[i];
  }

  //****************************************************************************
  public double GetDiff(int i)
  {
    return timx_diff1[i];
  }

  //****************************************************************************
  public String GetCalls(int i)
  {
    return cal[i];
  }

  //****************************************************************************
  public String GetLastError()
  {
    String err1=err;
    err="";
    return err1;
  }

  //****************************************************************************
  public String GetMin(int i)
  {
    return CMisc.align_double(timx_min[i], 5, 1);
  }

  //****************************************************************************
  public void MakeMinMain()
  {
    for (int i=0; i<nl; i++)
    {
      tim[i]=CMisc.align_double(timx_min[i], 5, 1);
    }
  }

  //****************************************************************************
  public void MakeMainMin()
  {
    for (int i=0; i<nl; i++)
    {
      double t1=0; 
      try {t1=Double.valueOf(tim[i]).doubleValue();}
      catch (NumberFormatException nfe1){}

      timx_min[i]=t1;
    }
  }

  //****************************************************************************
  public String GetBetter(int i)
  {
    String str1="";
    if (timx_better[i])
    {
      str1="(!!!"+CMisc.align_double(timx_diff[i], 5, 1)+")";
    }
    else
    {
      str1="            ";
    }  
    return str1;
  }

  //****************************************************************************
  public boolean isBetter(int i)
  {
    return timx_better[i];
  }

  //****************************************************************************
  public void CopyClass(CLLP cllp)
  {
    fn=cllp.fn;
    nl=cllp.nl;

    boolean xnew=false;

    if (tim==null)
    {
      xnew=true;

      tim=new String [nl];
      cal=new String [nl];

      timx_min   =new double[nl];
      timx_diff  =new double[nl];
      timx_diff1 =new double[nl];
      timx_better=new boolean[nl];
      timx_sel   =new boolean[nl];
    }

    for (int i=0; i<nl; i++)
    {
      tim[i]=cllp.tim[i];
      cal[i]=cllp.cal[i];
      
      timx_min[i]=cllp.timx_min[i];
      timx_diff[i]=cllp.timx_diff[i];
      timx_diff1[i]=cllp.timx_diff1[i];
      timx_better[i]=cllp.timx_better[i];
      if (xnew) timx_sel[i]=cllp.timx_sel[i];
    }
  }

  //****************************************************************************
  public void CleanBetter()
  {
    for (int i=0; i<nl; i++)
    {
      double t1=0; 
      try {t1=Double.valueOf(tim[i]).doubleValue();}
      catch (NumberFormatException nfe1){}

      timx_min[i]=t1;
      timx_diff[i]=0;
      timx_diff1[i]=0;
      timx_better[i]=false;
    }
  }
}
