//
// Class CTree for EOS tree files
//
// Copyright (C) 2000-2004 by Grigori G.Fursin
//
// http://homepages.inf.ed.ac.uk/gfursin
//
// ICSA, School of Informatics,
// University of Edinburgh, UK
//

//Import libraries...
import java.io.*;

public class CTree
{
  private String err;

  private String fname;

  private int nac;
  private int nac1;
  private int np;
  private int na;
  private int nl;

  private String[] fn;
  private String[] qp;
  private int[]    qpa;
  private int[]    qpl;

  private String[] qac;
  private int[]    qac1;
  private String[] qac2;

  private String[] qx;
  private String[] qx1;
  private String[] qx2;

  private String[] qa;
  private int[]    qa1;
  private String[] qa2;

  private String[] ql;
  private String[] ql0;
  private int[]    ql1;
  private String[] ql2;
  private String[] ql3;

  //****************************************************************************
  //"Initialization"
  //
  //Input:  fname1  = Tree filename
  //        fmax1   = Max number of active lines in the tree file
  //
  //Return: 0  - Ok
  //        >0 - Error
  //

  public int Init(String fname1,
                  int fmax1)
  {
    int i=0;

    if (fmax1<1) return 1;

    fname=fname1;

    nac=0;
    nac1=0;
    np=0;
    na=0;
    nl=0;

    return 0;
  }

  //****************************************************************************
  //"Parse Tree File"
  //
  //Return: 0  - Ok
  //        <0 - Error
  //
  public int Parse()
  {
    err="";

    BufferedReader fin=null;

    String str1="";

    String str1bb="";
    String str1bbb="";

    String str1a="";
    String str1b="";
    String str1c="";
    String str1d="";
    String str1e="";

    int ind1=0;
    int ind2=0;

    int xna=0;
    int xnl=0;

    nac=0;
    nac1=0;
    np=0;
    na=0;
    nl=0;

    //Reading tree (Pass 1)
    try
    {
      fin=new BufferedReader(new FileReader(fname));
    }
    catch (IOException e2)
    {
      err="Error: Can't find tree file !";
      return -1;
    }

    try
    {
      while ((str1=fin.readLine())!=null)
      {
        boolean incorrect=true;

        ind1=str1.indexOf("'");
        ind2=str1.indexOf("'", ind1+1);

        if ((ind1!=-1) && (ind2!=-1))
        {
          str1a=str1.substring(ind1+1, ind2);

          ind1=str1.indexOf("'", ind2+1);
          ind2=str1.indexOf("'", ind1+1);

          if ((ind1!=-1) && (ind2!=-1))
          {
            str1b=str1.substring(ind1+1, ind2);

            ind1=str1.indexOf("'", ind2+1);
            ind2=str1.indexOf("'", ind1+1);

            if ((ind1!=-1) && (ind2!=-1))
            {
              str1c=str1.substring(ind1+1, ind2);

              str1c=str1c.toLowerCase();

              incorrect=false;

              if (!str1b.equals(str1bb))
              {
                if (str1.charAt(0)=='+')
                {
                  xna=0;
                  xnl=0;
                  str1bb=str1b;
                  np++;    
                }
              }

              if (str1c.equals("array_comm"))
              {
                if (str1.charAt(0)=='+')
                {
                  nac++;
                }
                else
                {
                  nac1++;
                }
              }
              else if (str1c.equals("array_decl"))
              {
                if (str1.charAt(0)=='+')
                {
                  xna++;
                  if (xna>na) na=xna;
                }
              }
              else if (str1c.equals("loop"))
              {
                if (str1.charAt(0)=='+')
                {
                  xnl++;
                  if (xnl>nl) nl=xnl;
                }
              }
              else incorrect=true;
            }
          }
        }

        if (incorrect)
        {
          err="Error: Can't understand format of the tree file!";
          return -1;
        }
      }
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading tree file!";
      return -1;
    }

    try
    {
      fin.close();
    }
    catch (Exception e3)
    {}

    //Declaring arrays
    fn =new String [np+1];
    qp =new String [np+1];
    qpa=new int    [np+1];
    qpl=new int    [np+1];

    qac =new String [nac+1];
    qac1=new int    [nac+1];
    qac2=new String [nac+1];

    qx  =new String [nac1+1];
    qx1 =new String [nac1+1];
    qx2 =new String [nac1+1];

    qa =new String [np*na+1];
    qa1=new int    [np*na+1];
    qa2=new String [np*na+1];

    ql =new String [np*nl+1];
    ql0=new String [np*nl+1];
    ql1=new int    [np*nl+1];
    ql2=new String [np*nl+1];
    ql3=new String [np*nl+1];


    str1="";

    str1bb="";
    str1bbb="";

    str1a="";
    str1b="";
    str1c="";
    str1d="";
    str1e="";

    ind1=0;
    ind2=0;

    xna=0;
    xnl=0;

    nac=0;
    nac1=0;
    np=0;
    int np1=0;

    int lp=0;

    //Reading tree (Pass 2)
    try
    {
      fin=new BufferedReader(new FileReader(fname));
    }
    catch (IOException e2)
    {
      err="Error: Can't find tree file !";
      return -1;
    }

    try
    {
      while ((str1=fin.readLine())!=null)
      {
        boolean incorrect=true;

        ind1=str1.indexOf("'");
        ind2=str1.indexOf("'", ind1+1);

        if ((ind1!=-1) && (ind2!=-1))
        {
          str1a=str1.substring(ind1+1, ind2);
        
          ind1=str1.indexOf("'", ind2+1);
          ind2=str1.indexOf("'", ind1+1);

          if ((ind1!=-1) && (ind2!=-1))
          {
            str1b=str1.substring(ind1+1, ind2);

            ind1=str1.indexOf("'", ind2+1);
            ind2=str1.indexOf("'", ind1+1);

            if ((ind1!=-1) && (ind2!=-1))
            {
              str1c=str1.substring(ind1+1, ind2);

              ind1=str1.indexOf("'", ind2+1);
              ind2=str1.indexOf("'", ind1+1);

              if ((ind1!=-1) && (ind2!=-1))
              {
                str1d=str1.substring(ind1+1, ind2);

                str1c=str1c.toLowerCase();

                incorrect=false;

                if (!str1b.equals(str1bbb))
                {
                  lp=0; //reset loop count in each procedure
                  str1bbb=str1b;
                }

                if (!str1b.equals(str1bb))
                {
                  if (str1.charAt(0)=='+')
                  {
                    fn[np]=str1a;

                    if (np!=0)
                    {
                      qp[np1]=str1bb;
                      qpa[np1]=xna;
                      qpl[np1]=xnl;
                    }

                    xna=0;
                    xnl=0;
                    str1bb=str1b;
  
                    np++;
                    np1=np-1;
                  }
                }
  
                if (str1c.equals("array_comm"))
                {
                  if (str1.charAt(0)=='+')
                  {
                    qac[nac]=str1a;
                    qac1[nac]=np1;
                    qac2[nac]=str1d;
  
                    nac++;
                  }
                  else
                  {
                    qx[nac1]=str1a;
                    qx1[nac1]=str1b;
                    qx2[nac1]=str1d;
  
                    nac1++;
                  }
                }
                else if (str1c.equals("array_decl"))
                {
                  if (str1.charAt(0)=='+')
                  {
                    qa[np1*na+xna]=str1a;
                    qa1[np1*na+xna]=np1;
                    qa2[np1*na+xna]=str1d;
  
                    xna++;
                  }
                }
                else if (str1c.equals("loop"))
                {
                  lp++;

                  if (str1.charAt(0)=='+')
                  {
                    ind1=str1.indexOf("'", ind2+1);
                    ind2=str1.indexOf("'", ind1+1);
 
                    if ((ind1!=-1) && (ind2!=-1))
                    {
                      str1e=str1.substring(ind1+1, ind2);
        
                      ql [np1*nl+xnl]=str1a;
                      ql0[np1*nl+xnl]=String.valueOf(lp);
                      ql1[np1*nl+xnl]=np1;
                      ql2[np1*nl+xnl]=str1d;
                      ql3[np1*nl+xnl]=str1e;

                      xnl++;
                    }
                    else incorrect=true;
                  }
                }
                else incorrect=true;
              }
            }
          }
        }

        if (incorrect)
        {
          err="Error: Can't understand format of the tree file!";
          return -1;
        }
      }

      qp[np1]=str1bb;
      qpa[np1]=xna;
      qpl[np1]=xnl;
    }
    catch (Exception e1)
    {
      try
      {
        fin.close();
      }
      catch (Exception e2)
      {}
      err="Error: Problems while reading tree file!";
      return -1;
    }

    try
    {
      fin.close();
    }
    catch (Exception e3)
    {}

    return 0;
  }

  //****************************************************************************
  public String GetLastError()
  {
    String err1=err;
    err="";
    return err1;
  }

  //****************************************************************************
  public int GetNumArraysComm()
  {
    return nac;
  }

  //****************************************************************************
  public int GetNumArraysCommAll()
  {
    return nac1;
  }

  //****************************************************************************
  public int GetNumProcs()
  {
    return np;
  }

  //****************************************************************************
  public int GetNumArrays(int proc)
  {
    return qpa[proc];
  }

  //****************************************************************************
  public int GetNumLoops(int proc)
  {
    return qpl[proc];
  }

  //****************************************************************************
  public String GetFileAC(int ind)
  {
    return qac[ind];
  }

  //****************************************************************************
  public String GetProc(int ind)
  {
    return qp[ind];
  }

  //****************************************************************************
  public String GetProcAC(int ind)
  {
    return qp[qac1[ind]];
  }

  //****************************************************************************
  public String GetNameAC(int ind)
  {
    return qac2[ind];
  }

  //****************************************************************************
  public String GetFileACAll(int ind)
  {
    return qx[ind];
  }

  //****************************************************************************
  public String GetProcACAll(int ind)
  {
    return qx1[ind];
  }

  //****************************************************************************
  public String GetNameACAll(int ind)
  {
    return qx2[ind];
  }

  //****************************************************************************
  public String GetFileA(int proc, int ind)
  {
    return qa[proc*na+ind];
  }

  //****************************************************************************
  public String GetProcA(int proc, int ind)
  {
    return qp[qa1[proc*na+ind]];
  }

  //****************************************************************************
  public String GetNameA(int proc, int ind)
  {
    return qa2[proc*na+ind];
  }

  //****************************************************************************
  public String GetFileL(int proc, int ind)
  {
    return ql[proc*nl+ind];
  }

  //****************************************************************************
  public String GetProcL(int proc, int ind)
  {
    return qp[ql1[proc*nl+ind]];
  }

  //****************************************************************************
  public String GetNameL(int proc, int ind)
  {
    return ql2[proc*nl+ind];
  }

  //****************************************************************************
  public String GetCountL(int proc, int ind)
  {
    return ql0[proc*nl+ind];
  }

  //****************************************************************************
  public String GetParamL(int proc, int ind)
  {
    return ql3[proc*nl+ind];
  }

  //****************************************************************************
  public String GetFileName(int ind)
  {
    return fn[ind];
  }
}
